@@ -3,29 +3,20 @@ from __future__ import annotations
 
 import abc
 import base64
+import binascii
 import dataclasses
 import enum
 import functools
 import io
-import sys
 from dataclasses import InitVar
-from typing import Iterator
-from typing import Literal
-from typing import Optional
-from typing import Tuple
-from typing import Union
+from typing import Iterator, Literal, Optional, Tuple, Union
 
 import picharsso
 import PIL.Image
 from picharsso.draw import gradient
 from PIL.Image import Image
-from rich import ansi
-from rich import measure
-from rich import style
-from rich import text
-from rich.console import Console
-from rich.console import ConsoleOptions
-from rich.console import RenderResult
+from rich import ansi, measure, style, text
+from rich.console import Console, ConsoleOptions, RenderResult
 from rich.measure import Measurement
 from rich.text import Text
 
@@ -38,49 +29,86 @@ except ModuleNotFoundError:
     pass
 
 
-def render_drawing(
-    data: Data,
-    image_drawing: Literal["block", "character", "braille", None],
+def _get_image(data: Data, image_type: str) -> Union[bytes, None]:
+    """Extract an image in bytes from data."""
+    encoded_image = data[image_type]
+    decoded_image: Union[bytes, None]
+    try:
+        decoded_image = base64.b64decode(encoded_image)
+    except binascii.Error:
+        decoded_image = None
+    return decoded_image
+
+
+def _get_fallback_text(data: Data) -> str:
+    """Get a fallback text from data."""
+    return data.get("text/plain", "Image")
+
+
+def choose_drawing(
+    image: Union[bytes, None],
+    fallback_text: str,
     image_type: str,
-    unicode: bool,
+    image_drawing: Literal["block", "character", "braille"],
     color: bool,
     negative_space: bool,
-    characters: str = gradient.DEFAULT_CHARSET,
+    characters: Optional[str] = None,
 ) -> Union[Drawing, None]:
-    """Render a drawing of an image."""
+    """Choose which drawing to render an image with."""
     rendered_image: Drawing
-    if image_type != "image/svg+xml":
-        if (
-            image_drawing == "block"
-            and unicode
-            and "terminedia" in sys.modules
-            and color
-        ):
-            rendered_image = UnicodeDrawing.from_data(data, image_type=image_type)
+    if image is not None and image_type != "image/svg+xml":
+        if image_drawing == "block":
+            rendered_image = UnicodeDrawing(image=image, fallback_text=fallback_text)
             return rendered_image
 
-        elif image_drawing == "braille" and unicode:
-            rendered_image = BrailleDrawing.from_data(
-                data,
-                image_type=image_type,
+        elif image_drawing == "braille":
+            rendered_image = BrailleDrawing(
+                image=image,
+                fallback_text=fallback_text,
                 color=color,
             )
             return rendered_image
-
         elif image_drawing == "character":
-            rendered_image = CharacterDrawing.from_data(
-                data,
-                image_type=image_type,
+            rendered_image = CharacterDrawing(
+                image=image,
+                fallback_text=fallback_text,
                 color=color,
                 negative_space=negative_space,
                 characters=characters,
             )
             return rendered_image
-
+        else:
+            raise ValueError(
+                f"{image_drawing} is an invalid image_drawing,"
+                " expected 'block', 'character', or 'braille'"
+            )
     return None
 
 
-class Bottleneck(str, enum.Enum):
+def render_drawing(
+    data: Data,
+    image_drawing: Literal["block", "character", "braille"],
+    image_type: str,
+    color: bool,
+    negative_space: bool,
+    characters: Optional[str] = None,
+) -> Union[Drawing, None]:
+    """Render a drawing of an image."""
+    image = _get_image(data, image_type=image_type)
+    fallback_text = _get_fallback_text(data)
+    rendered_drawing = choose_drawing(
+        image=image,
+        fallback_text=fallback_text,
+        image_drawing=image_drawing,
+        image_type=image_type,
+        color=color,
+        negative_space=negative_space,
+        characters=characters,
+    )
+    return rendered_drawing
+
+
+class Bottleneck(enum.Enum):
     """The bottleneck when rendering a drawing."""
 
     WIDTH = enum.auto()
@@ -94,9 +122,10 @@ def _detect_image_bottleneck(
     image_height: int,
     max_width: Union[int, None],
     max_height: Union[int, None],
+    scaling_factor: float = 2.125,
 ) -> Bottleneck:
     """Detect which dimension the image is bottlenecked on."""
-    image_ratio = image_width / image_height
+    image_ratio = scaling_factor * image_width / image_height
     max_ratio = (
         max_width / max_height
         if max_width is not None and max_height is not None
@@ -125,25 +154,24 @@ class DrawingDimension:
     image: InitVar[Image]
     max_width: Optional[int] = None
     max_height: Optional[int] = None
+    scaling_factor: float = 2.125
 
     def __post_init__(self, image: Image) -> None:
         """Constructor."""
         image_width, image_height = image.size
-        image_ratio = image_width / image_height
+        image_ratio = self.scaling_factor * image_width / image_height
 
         self.bottleneck = _detect_image_bottleneck(
             image_width=image_width,
             image_height=image_height,
             max_width=self.max_width,
             max_height=self.max_height,
+            scaling_factor=self.scaling_factor,
         )
 
-        image_width, image_height = image.size
-        image_ratio = image_width / image_height
-
         if self.bottleneck == Bottleneck.WIDTH and self.max_width is not None:
             drawing_width = self.max_width
-            drawing_height = int(drawing_width / image_ratio)
+            drawing_height = round(drawing_width / image_ratio)
         elif self.bottleneck == Bottleneck.HEIGHT and self.max_height is not None:
             drawing_height = self.max_height
             drawing_width = int(drawing_height * image_ratio)
@@ -214,7 +242,19 @@ def _render_block_drawing(
         dimensions = DrawingDimension(
             pil_image, max_width=max_width, max_height=max_height
         )
-        size = terminedia.V2(x=dimensions.drawing_width, y=dimensions.drawing_height)
+        character_dimensions = CharacterDimensions(
+            bottleneck=dimensions.bottleneck, max_width=max_width, max_height=max_height
+        )
+        # terminedia takes a scaled versions of y
+        scaled_y = (
+            character_dimensions.height * dimensions.scaling_factor
+            if character_dimensions.height is not None
+            else None
+        )
+        size = terminedia.V2(
+            x=character_dimensions.width,
+            y=scaled_y,
+        )
 
         shape = terminedia.shape(
             pil_image,
@@ -222,6 +262,7 @@ def _render_block_drawing(
             promote=True,
             resolution="square",
         )
+        pil_image.close()
 
         output = io.StringIO()
         shape.render(output=output, backend="ANSI")
@@ -265,7 +306,7 @@ class UnicodeDrawing(Drawing):
             max_width=options.max_width,
             fallback_text=self.fallback_text,
         )
-        minimum = max(len(line) for line in rendered_unicode_drawing)
+        minimum = max(line.cell_len for line in rendered_unicode_drawing)
         return measure.Measurement(minimum, maximum=options.max_width)
 
 
@@ -274,16 +315,16 @@ class CharacterDimensions:
     """Dimensions for a character drawing."""
 
     bottleneck: Bottleneck
-    max_width: int
-    max_height: int
+    max_width: Union[int, None]
+    max_height: Union[int, None]
 
     def __post_init__(self) -> None:
         """Constructor."""
         if self.bottleneck == Bottleneck.WIDTH:
             width = self.max_width
-            height = 0
+            height = None
         elif self.bottleneck == Bottleneck.HEIGHT:
-            width = 0
+            width = None
             height = self.max_height
         else:
             width = self.max_width
@@ -300,11 +341,12 @@ def _render_character_drawing(
     max_width: int,
     max_height: int,
     fallback_text: str,
-    characters: str = gradient.DEFAULT_CHARSET,
+    characters: Optional[str] = None,
     negative_space: bool = True,
 ) -> Tuple[Text, ...]:
     """Render a representation of an image with text characters."""
     rendered_character_drawing: Tuple[Text, ...]
+    characters = characters if characters is not None else gradient.DEFAULT_CHARSET
     try:
         pil_image = PIL.Image.open(io.BytesIO(image))
 
@@ -318,16 +360,16 @@ def _render_character_drawing(
         character_dimensions = CharacterDimensions(
             bottleneck=dimensions.bottleneck, max_width=max_width, max_height=max_height
         )
-
         drawer = picharsso.new_drawer(
             style="gradient",
-            width=character_dimensions.width,
-            height=character_dimensions.height,
+            width=character_dimensions.width or 0,
+            height=character_dimensions.height or 0,
             colorize=color,
             charset=characters,
             negative=negative_space,
         )
         drawing = drawer(pil_image)
+        pil_image.close()
 
         decoder = ansi.AnsiDecoder()
         rendered_character_drawing = tuple(decoder.decode(drawing))
@@ -344,7 +386,7 @@ class CharacterDrawing(Drawing):
         fallback_text: str,
         color: bool,
         negative_space: bool,
-        characters: str = gradient.DEFAULT_CHARSET,
+        characters: Optional[str] = None,
     ) -> None:
         """Constructor."""
         super().__init__(image=image, fallback_text=fallback_text)
@@ -369,7 +411,7 @@ class CharacterDrawing(Drawing):
         image_type: str,
         color: bool,
         negative_space: bool,
-        characters: str = gradient.DEFAULT_CHARSET,
+        characters: Optional[str] = None,
     ) -> CharacterDrawing:
         """Create a drawing from notebook data."""
         encoded_image = data[image_type]
@@ -411,7 +453,7 @@ class CharacterDrawing(Drawing):
             fallback_text=self.fallback_text,
             negative_space=self.negative_space,
         )
-        minimum = max(len(line) for line in rendered_character_drawing)
+        minimum = max(line.cell_len for line in rendered_character_drawing)
         return measure.Measurement(minimum=minimum, maximum=options.max_width)
 
 
@@ -422,7 +464,6 @@ def _render_braille_drawing(
     max_width: int,
     max_height: int,
     fallback_text: str,
-    negative_space: bool = True,
 ) -> Tuple[Text, ...]:
     """Render a representation of an image with braille characters."""
     rendered_character_drawing: Tuple[Text, ...]
@@ -441,11 +482,12 @@ def _render_braille_drawing(
         )
         drawer = picharsso.new_drawer(
             style="braille",
-            width=character_dimensions.width,
-            height=character_dimensions.height,
+            width=character_dimensions.width or 0,
+            height=character_dimensions.height or 0,
             colorize=color,
         )
         drawing = drawer(pil_image)
+        pil_image.close()
 
         decoder = ansi.AnsiDecoder()
         rendered_character_drawing = tuple(decoder.decode(drawing))
@@ -515,5 +557,5 @@ class BrailleDrawing(Drawing):
             max_height=options.max_height,
             fallback_text=self.fallback_text,
         )
-        minimum = max(len(line) for line in rendered_braille_drawing)
+        minimum = max(line.cell_len for line in rendered_braille_drawing)
         return measure.Measurement(minimum=minimum, maximum=options.max_width)
