@@ -2,25 +2,17 @@
 import dataclasses
 import itertools
 from dataclasses import InitVar
-from typing import Iterator
-from typing import List
-from typing import Literal
-from typing import Optional
-from typing import Tuple
-from typing import Union
+from typing import Iterator, List, Literal, Optional, Tuple, Union
 
 from nbformat import NotebookNode
 from rich import padding
-from rich.padding import Padding
-from rich.padding import PaddingDimensions
+from rich.padding import Padding, PaddingDimensions
 
 from nbpreview.component.content import input
 from nbpreview.component.content.input import Cell
-from nbpreview.component.content.output import error
-from nbpreview.component.content.output import Output
-from nbpreview.component.content.output import result
-from nbpreview.component.content.output import stream
-from nbpreview.component.content.output.result import execution_indicator
+from nbpreview.component.content.output import error, stream
+from nbpreview.component.content.output.output import Output
+from nbpreview.component.content.output.result import execution_indicator, result
 from nbpreview.component.content.output.result.execution_indicator import Execution
 
 Content = Union[Cell, Padding]
@@ -72,8 +64,18 @@ def render_input_row(
     pad: Tuple[int, int, int, int],
     language: str,
     theme: str,
+    nerd_font: bool,
+    unicode: bool,
+    images: bool,
+    image_drawing: Literal["block", "character", "braille"],
+    color: bool,
+    negative_space: bool,
+    hyperlinks: bool,
+    files: bool,
+    hide_hyperlink_hints: bool,
+    characters: Optional[str] = None,
     unicode_border: Optional[bool] = None,
-) -> Row:
+) -> Union[Row, None]:
     """Render a Jupyter Notebook cell.
 
     Args:
@@ -86,13 +88,32 @@ def render_input_row(
         theme (str): The theme to use for syntax highlighting. May be
             "ansi_light", "ansi_dark", or any Pygments theme. By default
             "ansi_dark".
+        nerd_font (bool): Whether to use nerd fonts as an icon.
+        unicode (bool): Whether to use unicode characters as an icon.
+        images (bool): Whether to render images in the output.
+        image_drawing (Literal["block", "character", "braille"]):
+            The type of characters to draw images with.
+        color (bool):
+            Whether to draw images using color.
+        negative_space (bool):
+            Whether to draw only the dark areas of an image with
+                characters.
+        hyperlinks (bool):
+            Whether to render terminal hyperlinks.
+        files (bool):
+            Whether to write temporary files.
+        hide_hyperlink_hints (bool):
+            Whether to hide hyperlink hints.
+        characters (str):
+            The characters to draw images with. If set to None will
+            default to ' :!?PG@'.
         unicode_border (Optional[bool]): Whether to render the cell
             borders using unicode characters. Will autodetect by
             default.
 
     Returns:
-        Row: The execution count indicator and cell
-            content.
+        Row: The execution count indicator and cell content if cell type
+            is known, else None.
     """
     cell_type = cell.get("cell_type")
     source = cell.source
@@ -102,7 +123,21 @@ def render_input_row(
     execution: Union[Execution, None] = None
     top_pad = not plain
     if cell_type == "markdown":
-        rendered_cell = input.MarkdownCell(source, theme=theme, pad=pad)
+        rendered_cell = input.MarkdownCell(
+            source,
+            theme=theme,
+            pad=pad,
+            nerd_font=nerd_font,
+            unicode=unicode,
+            images=images,
+            image_drawing=image_drawing,
+            color=color,
+            negative_space=negative_space,
+            hyperlinks=hyperlinks,
+            files=files,
+            hide_hyperlink_hints=hide_hyperlink_hints,
+            characters=characters,
+        )
 
     elif cell_type == "code":
         execution = Execution(cell.execution_count, top_pad=top_pad)
@@ -114,10 +149,12 @@ def render_input_row(
             default_lexer_name=default_lexer_name,
         )
 
-    # Includes cell_type == "raw"
-    else:
+    elif cell_type == "raw":
         rendered_cell = Cell(source, plain=plain, safe_box=safe_box)
 
+    else:
+        return None
+
     cell_row = Row(rendered_cell, plain=plain, execution=execution)
     return cell_row
 
@@ -133,7 +170,7 @@ def render_output_row(
     theme: str,
     pad: PaddingDimensions,
     images: bool,
-    image_drawing: Literal["block", "character", "braille", None],
+    image_drawing: Literal["block", "character", "braille"],
     color: bool,
     negative_space: bool,
 ) -> Iterator[OutputRow]:
